<?php
declare(strict_types=1);

namespace FiloBlu\Antifraud\Observer\Adyen;

use Magento\Sales\Model\Order;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

class NotificationProcessed implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Psr\Log\LoggerInterface $logger
     */
    protected $logger;

    protected $config;

    const MANAGED_NOTIFICATION_CODES = array('AUTHORISATION');
    const ENABLE_FRAUD_SCORE_CHECK_CONFIG_PATH = 'filoblu_antifraud/antifraud/enable_fraud_score';
    const FRAUD_SCORE_THRESHOLD_CONFIG_PATH = 'filoblu_antifraud/antifraud/fraud_score';
    const ENABLE_GRAND_TOTAL_CHECK_CONFIG_PATH = 'filoblu_antifraud/antifraud/enable_grandtotal_check';
    const GRAND_TOTAL_THRESHOLD_CONFIG_PATH = 'filoblu_antifraud/antifraud/grand_total';
    const DEBUG_LOG_ACTIVE = true;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        ScopeConfigInterface $config
    ) {
        $this->logger = $logger;
        $this->config = $config;
    }
    /**
     * Execute observer
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(
        \Magento\Framework\Event\Observer $observer
    ) {
        /* @var $order Order */
        if ($this->isDebugLogActive()) {
            $this->logger->addInfo("FiloBlu\Antifraud\Observer\Adyen\NotificationProcessed: Processing Notification");
        }
        $notification_code = $observer->getEvent()->getNotificationCode();
        if (!in_array($notification_code, self::MANAGED_NOTIFICATION_CODES)) {
            if ($this->isDebugLogActive()) {
                $this->logger->addInfo("FiloBlu\Antifraud\Observer\Adyen\NotificationProcessed: Skipped notification");
            }
            return;
        }
        if ($notification_code == 'AUTHORISATION' && $observer->getEvent()->getManualReview()) {
            if ($this->isDebugLogActive()) {
                $this->logger->addInfo("FiloBlu\Antifraud\Observer\Adyen\NotificationProcessed: Skipped notification due to Manual review flag");
            }
            return;
        }
        $order = $observer->getEvent()->getOrder();
        if ($this->config->isSetFlag(self::ENABLE_FRAUD_SCORE_CHECK_CONFIG_PATH, ScopeInterface::SCOPE_STORE) == true) {
            $payment_additional_information = $order->getPayment()->getAdditionalInformation();
            if (isset($payment_additional_information['adyen_total_fraud_score']) && !empty($payment_additional_information['adyen_total_fraud_score'])
                && is_numeric($this->config->getValue(self::FRAUD_SCORE_THRESHOLD_CONFIG_PATH, ScopeInterface::SCOPE_STORE))
            ) {
                $order_fraud_score = $payment_additional_information['adyen_total_fraud_score'];
                if ($order_fraud_score >= $this->config->getValue(self::FRAUD_SCORE_THRESHOLD_CONFIG_PATH, ScopeInterface::SCOPE_STORE)) {
                    if ($this->isDebugLogActive()) {
                        $this->logger->addInfo("FiloBlu\Antifraud\Observer\Adyen\NotificationProcessed: Holding order due to fraud score");
                    }
                    $order->addCommentToStatusHistory(__('FiloBlu Antifraud: Holding order due to fraud score'));
                    $this->holdOrder($order);
                    return;
                }
            }
        }
        if ($this->config->isSetFlag(self::ENABLE_GRAND_TOTAL_CHECK_CONFIG_PATH)) {
            $grand_total = $order->getGrandTotal();
            if (is_numeric($this->config->getValue(self::GRAND_TOTAL_THRESHOLD_CONFIG_PATH)) && $grand_total >= $this->config->getValue(self::GRAND_TOTAL_THRESHOLD_CONFIG_PATH)) {
                if ($this->isDebugLogActive()) {
                    $this->logger->addInfo("FiloBlu\Antifraud\Observer\Adyen\NotificationProcessed: Holding order due to high grand total");
                }
                $order->addCommentToStatusHistory(__('FiloBlu Antifraud: Holding order due to high grand total'));
                $this->holdOrder($order);
                return;
            }
        }
        
    }

    /**
     * @param Order $order
     */
    protected function holdOrder(Order $order) 
    {
        $order->hold();
    }


    protected function isDebugLogActive() 
    {
        return self::DEBUG_LOG_ACTIVE;
    }
}

